# CONTENT:
# - classes (operators, property groups etc.) 
#   - main addon program
#   - gui 
bl_info = {
    "name": "Demo s loggerem 3",
    "description": "Demonstrační modul s loggerem 3.",
    "author": "Jiří Chludil",
    "version": (1, 0, 0),
    "blender": (2, 93, 5),
    "category": "Add Mesh"
}

import bpy, os, sys
from random import uniform, randint
from bpy.props import *
import random
import requests
import json
import time
import bmesh
from bpy_extras import object_utils

# -----------------------------------------------------------------------

class House(bpy.types.PropertyGroup):
  
    name : bpy.props.StringProperty(name = "")
    has_chimney: bpy.props.BoolProperty()
    id: bpy.props.IntProperty()
    
# -----------------------------------------------------------------------

class HouseGroup(bpy.types.PropertyGroup):
    coll : bpy.props.CollectionProperty(type=House)
    index : bpy.props.IntProperty() 

# -----------------------------------------------------------------------

class HouseList(bpy.types.UIList):

    def draw_item(self, context, layout, data, item, icon, active_data, active_propname, index):

        if self.layout_type in {'DEFAULT', 'COMPACT'}:
            layout.prop(item,"name", emboss=False, icon='QUESTION')   
            if(item.has_chimney):
                op = layout.operator("object.remove_chimney")    
                op.current_item = item.name
            else:
                op = layout.operator("object.add_chimney")
                op.current_item = item.name
        elif self.layout_type in { 'GRID'}:
            layout.alignment = 'CENTER'
            layout.label(text="", icon_value=icon)
            
 # -----------------------------------------------------------------------     
    
class UpdateHouseList(bpy.types.Operator):
    
    bl_idname = "house.update_list"
    bl_label = "Aktualizuj seznam domu"
    
    def execute(self, context):
        scene = context.scene
        scene.prop_group_info.coll.clear()
        
        for obj in scene.objects: 
            
            if 'data_type' in obj.data.keys() and obj.data['data_type'] == "HOUSE":
                item = scene.prop_group_info.coll.add()
                item.name = obj.name
                item.has_chimney = obj.data['data_chimney']
            print(obj.name)
            print(obj.type)
            print(obj.data)
            print(obj.data.keys())
                
        scene.prop_group_info.index = len(scene.prop_group_info.coll)
        
        return {'FINISHED'}    

# -----------------------------------------------------------------------

class DemoPanel(bpy.types.Panel):
    """Class creating a Panel in Tool properties."""

    bl_label = "Demo"
    bl_idname = "demo"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Tool"
    bl_context = "objectmode"

    def draw(self, context):
        layout = self.layout
        scene = context.scene
        house = scene.house
    
        row = layout.row(align=True) 
        row.prop(house, "size")
        row = layout.row(align=True) 
        row.prop(house, "roof")
        row = layout.row(align=True) 
        row.operator("object.add_house")
        row.operator("object.edit_house")
        row = layout.row(align=True) 
        
        # Informační okno
        layout.template_list("HouseList", "", scene.prop_group_info, "coll", \
            scene.prop_group_info, "index", rows=5, maxrows=5, columns=3, type='DEFAULT')
        row = layout.row(align=True) 
        row.operator("house.update_list")  
        try:
            current_item = scene.prop_group_info.coll[scene.prop_group_info.index].name
            obj = scene.objects[current_item]
            scene.house.size.x = obj.data['data_size_x']
            scene.house.size.y = obj.data['data_size_y']
            scene.house.size.z = obj.data['data_size_z']
            scene.house.roof = obj.data['data_roof']
            
            
        except IndexError:
            current_item = ""
                    
        row = layout.row(align=True)  
        row.label(text="Active Item is: {}".format(current_item))
     
     
       
class HouseData(bpy.types.PropertyGroup):
   
    size : bpy.props.FloatVectorProperty(
        name = 'Size',
        description = 'Size of base house',
        subtype = 'XYZ',
        default=(1.0, 1.0, 1.0),
    )
    
    roof : bpy.props.FloatProperty(
       name = 'Roof',
       description = 'Height of roof',
       default = 1,
       min = 0,
       max = 2,
       step = 25,
    )



class RemoveChimney(bpy.types.Operator):  
    bl_idname = "object.remove_chimney"
    bl_label = "Remove Chimney"
    bl_options = {'REGISTER', 'UNDO'}
    
    current_item = bpy.props.StringProperty()  
  
    def execute(self, context):
        return {'FINISHED'} 
    
class AddChimney(bpy.types.Operator):  
    bl_idname = "object.add_chimney"
    bl_label = "Add Chimney"
    bl_options = {'REGISTER', 'UNDO'}
  
    current_item = bpy.props.StringProperty()  
  
    def execute(self, context):
        try:
            scene = context.scene
            obj = scene.objects[self.current_item]
            
            bm = bmesh.new()
            bm.from_mesh(obj.data)
            for f in bm.faces:
                f.select = False;
            bm.faces.ensure_lookup_table()
            bm.faces[5].select = True    
            bm.to_mesh(obj.data)
            bm.free()
            
            print("MARK 1 - SELECT ROOF")
            bpy.context.view_layer.objects.active = obj
            bpy.ops.object.mode_set(mode='EDIT')
            bpy.ops.mesh.subdivide()
            bpy.ops.object.mode_set(mode='OBJECT')
            print("MARK 2 - SUBDIVIDE")
            
            bm = bmesh.new()
            bm.from_mesh(obj.data)
            for f in bm.faces:
                f.select = False;
            bm.faces.ensure_lookup_table()
            bm.faces[5].select = True    
            bm.to_mesh(obj.data)
            bm.free()
            
            print("MARK 3 - SELECT ROOF")    
            bpy.ops.object.mode_set(mode='EDIT')
            bpy.ops.mesh.subdivide()
            bpy.ops.object.mode_set(mode='OBJECT')
            print("MARK 4 - SUBDIVIDE")
            
            bm = bmesh.new()
            bm.from_mesh(obj.data)
            bm.faces.ensure_lookup_table()
            bm.faces.remove(bm.faces[13])
            nverts = [(0,-0.25,2),(0,-0.125,2),(0.25,-0.25,2),(0.25,-0.125,2)]
            for v in nverts:
                bm.verts.new(v)
            bm.verts.ensure_lookup_table()
            bm.faces.new([bm.verts[22],bm.verts[23],bm.verts[21],bm.verts[20]])
            bm.faces.new([bm.verts[23],bm.verts[22],bm.verts[18],bm.verts[19]])
            bm.faces.new([bm.verts[21],bm.verts[23],bm.verts[19],bm.verts[17]])
            bm.faces.new([bm.verts[20],bm.verts[21],bm.verts[17],bm.verts[14]])
            bm.faces.new([bm.verts[22],bm.verts[20],bm.verts[14],bm.verts[18]])
            print("MARK 5 - Chimney")
            for f in bm.faces:
                f.select = False;  
            bm.to_mesh(obj.data)
            bm.free()
             
            obj.data['data_chimney'] = True
            
           
            
        except IndexError:
            current_item = ""
        return {'FINISHED'} 
              


          
class EditHouse(bpy.types.Operator):  
    bl_idname = "object.edit_house"
    bl_label = "Edit House"
    bl_options = {'REGISTER', 'UNDO'}
  
    def execute(self, context):
        try:
            scene = context.scene
            current_item = scene.prop_group_info.coll[scene.prop_group_info.index].name
            obj = scene.objects[current_item]
            
            bm = bmesh.new()
            bm.from_mesh(obj.data)
            print(obj.name)
            for count, v in enumerate(bm.verts):
                print(count)
                print(str(v.co[0])+" "+str(v.co[1])+" "+str(v.co[2]))
            print("MARK1")
            bm.verts.ensure_lookup_table()
            bm.verts[8].co.z = 10.0
            bm.verts[9].co.z = 10.0
            print("MARK2")
            bm.to_mesh(obj.data)
            bm.free()
            
        except IndexError:
            current_item = ""
        return {'FINISHED'} 
          
          
class AddHouse(bpy.types.Operator):  
    bl_idname = "object.add_house"
    bl_label = "Add House"
    bl_options = {'REGISTER', 'UNDO'}
  
    def execute(self, context):
        print("Startuji")   
        verts, edges, faces = self.add_house(context.scene.house.size, context.scene.house.roof)
        print("Mesh vygenerovan")   
        obj = self.create_mesh_object(context, verts, edges, faces, "House") 
        print("Domecek postaven")    
        obj.data['data_type'] = "HOUSE"
        obj.data['data_size_x'] = context.scene.house.size[0]
        obj.data['data_size_y'] = context.scene.house.size[1]
        obj.data['data_size_z'] = context.scene.house.size[2]
        obj.data['data_roof'] = context.scene.house.roof
        obj.data['data_chimney'] = False
        print("Domecek hotov")    
        return {'FINISHED'}
        
    def add_house(self, size = (1.0 ,1.0 ,1.0 ), roof = 1):
        verts = []
        edges = []
        faces = []
        size_x = size[0] / 2.0
        size_y = size[1] / 2.0
        size_z = size[2] / 2.0
        verts.append((-size_x, -size_y, -size_z))
        verts.append(( size_x, -size_y, -size_z))
        verts.append(( size_x, size_y, -size_z))
        verts.append((-size_x, size_y, -size_z))
        verts.append((-size_x, -size_y, size_z))
        verts.append(( size_x, -size_y, size_z))
        verts.append(( size_x, size_y, size_z))
        verts.append((-size_x, size_y, size_z))
        verts.append((-size_x, 0, size_z+roof))
        verts.append(( size_x, 0, size_z+roof))
        faces.append([0,3,2,1])
        faces.append([0,1,5,4])
        faces.append([1,2,6,5])
        faces.append([2,3,7,6])
        faces.append([3,0,4,7])
        faces.append([4,5,9,8])
        faces.append([6,7,8,9])
        faces.append([5,6,9])
        faces.append([4,8,7])
        return verts, edges, faces

    def create_mesh_object(self, context, verts, edges, faces, name):
        mesh = bpy.data.meshes.new(name)
        mesh.from_pydata(verts, edges, faces)
        mesh.update()
        
        return object_utils.object_data_add(context, mesh, operator=None)      
    
        
#def menu_func(self, context):
#    return
    
        
classes = [DemoPanel, UpdateHouseList, House, HouseGroup, HouseList, HouseData, AddHouse, EditHouse, AddChimney, RemoveChimney]

def menu_func(self, context):
    self.layout.operator(DemoPanel.bl_idname)

def register():
    from bpy.utils import register_class
    for cls in classes:
       register_class(cls)
    bpy.types.Scene.house = PointerProperty(type=HouseData)
    bpy.types.Scene.prop_group_info = PointerProperty(type=HouseGroup)   
   #bpy.types.VIEW3D_MT_mesh_add.append(menu_func)       

def unregister():
    from bpy.utils import unregister_class
    for cls in reversed(classes):
       unregister_class(cls)
    del bpy.types.Scene.prop_group_info
    del bpy.types.Scene.items    
    #bpy.types.VIEW3D_MT_mesh_add.remove(menu_func)

if __name__ == "__main__":
    register()
    #unregister()
    
